@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2012 Conrad Sanderson
+// Copyright (C) 2008-2015 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2015 Conrad Sanderson
 // Copyright (C) 2009-2010 Ian Cullinan
 // Copyright (C) 2012 Ryan Curtin
 // Copyright (C) 2013 Szabolcs Horvat
@@ -704,7 +704,7 @@ diskio::gen_tmp_name(const std::string& x)
   
   for(uword i=0; i<x_size; ++i)
     {
-    sum += u8(x[i]);
+    sum = (sum + u8(x[i])) & 0xff;
     }
   
   conv_to_hex(&tmp[char_count], sum);
@@ -913,7 +913,7 @@ diskio::save_raw_ascii(const Mat<eT>& x, std::ostream& f)
         f.width(cell_width);
         }
       
-      f << x.at(row,col);
+      arma_ostream::print_elem(f, x.at(row,col), false);
       }
       
     f.put('\n');
@@ -1039,7 +1039,7 @@ diskio::save_arma_ascii(const Mat<eT>& x, std::ostream& f)
         f.width(cell_width);
         }
       
-      f << x.at(row,col);
+      arma_ostream::print_elem(f, x.at(row,col), false);
       }
     
     f.put('\n');
@@ -1111,7 +1111,7 @@ diskio::save_csv_ascii(const Mat<eT>& x, std::ostream& f)
     {
     for(uword col=0; col < x_n_cols; ++col)
       {
-      f << x.at(row,col);
+      arma_ostream::print_elem(f, x.at(row,col), false);
       
       if( col < (x_n_cols-1) )
         {
@@ -1296,44 +1296,44 @@ diskio::save_hdf5_binary(const Mat<eT>& x, const std::string& final_name)
     #if !defined(ARMA_PRINT_HDF5_ERRORS)
       {
       // Disable annoying HDF5 error messages.
-      H5Eset_auto(H5E_DEFAULT, NULL, NULL);
+      arma_H5Eset_auto(H5E_DEFAULT, NULL, NULL);
       }
     #endif
-
+    
     bool save_okay = false;
     
     const std::string tmp_name = diskio::gen_tmp_name(final_name);
     
     // Set up the file according to HDF5's preferences  
-    hid_t file = H5Fcreate(tmp_name.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+    hid_t file = arma_H5Fcreate(tmp_name.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
     
     // We need to create a dataset, datatype, and dataspace
     hsize_t dims[2];
     dims[1] = x.n_rows;
     dims[0] = x.n_cols;
     
-    hid_t dataspace = H5Screate_simple(2, dims, NULL);   // treat the matrix as a 2d array dataspace
+    hid_t dataspace = arma_H5Screate_simple(2, dims, NULL);   // treat the matrix as a 2d array dataspace
     hid_t datatype  = hdf5_misc::get_hdf5_type<eT>();
-
+    
     // If this returned something invalid, well, it's time to crash.
     arma_check(datatype == -1, "Mat::save(): unknown datatype for HDF5");
-
+    
     // MATLAB forces the users to specify a name at save time for HDF5; Octave
     // will use the default of 'dataset' unless otherwise specified, so we will
     // use that.
-    hid_t dataset = H5Dcreate(file, "dataset", datatype, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    hid_t dataset = arma_H5Dcreate(file, "dataset", datatype, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
     
     // H5Dwrite does not make a distinction between row-major and column-major;
     // it just writes the memory.  MATLAB and Octave store HDF5 matrices as
     // column-major, though, so we can save ours like that too and not need to
     // transpose.
-    herr_t status = H5Dwrite(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, x.mem);
+    herr_t status = arma_H5Dwrite(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, x.mem);
     save_okay = (status >= 0);
     
-    H5Dclose(dataset);
-    H5Tclose(datatype);
-    H5Sclose(dataspace);
-    H5Fclose(file);
+    arma_H5Dclose(dataset);
+    arma_H5Tclose(datatype);
+    arma_H5Sclose(dataspace);
+    arma_H5Fclose(file);
     
     if(save_okay == true) { save_okay = diskio::safe_rename(tmp_name, final_name); }
     
@@ -2030,16 +2030,16 @@ diskio::load_hdf5_binary(Mat<eT>& x, const std::string& name, std::string& err_m
     #if !defined(ARMA_PRINT_HDF5_ERRORS)
       {
       // Save old error handler.
-      H5Eget_auto(H5E_DEFAULT, &old_func, &old_client_data);
+      arma_H5Eget_auto(H5E_DEFAULT, &old_func, &old_client_data);
 
       // Disable annoying HDF5 error messages.
-      H5Eset_auto(H5E_DEFAULT, NULL, NULL);
+      arma_H5Eset_auto(H5E_DEFAULT, NULL, NULL);
       }
     #endif
 
     bool load_okay = false;
     
-    hid_t fid = H5Fopen(name.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
+    hid_t fid = arma_H5Fopen(name.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
     
     if(fid >= 0)
       {
@@ -2054,27 +2054,27 @@ diskio::load_hdf5_binary(Mat<eT>& x, const std::string& name, std::string& err_m
 
       if(dataset >= 0)
         {
-        hid_t filespace = H5Dget_space(dataset);
+        hid_t filespace = arma_H5Dget_space(dataset);
         
         // This must be <= 2 due to our search rules.
-        const int ndims = H5Sget_simple_extent_ndims(filespace);
+        const int ndims = arma_H5Sget_simple_extent_ndims(filespace);
         
         hsize_t dims[2];
-        const herr_t query_status = H5Sget_simple_extent_dims(filespace, dims, NULL);
+        const herr_t query_status = arma_H5Sget_simple_extent_dims(filespace, dims, NULL);
         
         // arma_check(query_status < 0, "Mat::load(): cannot get size of HDF5 dataset");
         if(query_status < 0)
           {
           err_msg = "cannot get size of HDF5 dataset in ";
           
-          H5Sclose(filespace);
-          H5Dclose(dataset);
-          H5Fclose(fid);
+          arma_H5Sclose(filespace);
+          arma_H5Dclose(dataset);
+          arma_H5Fclose(fid);
     
           #if !defined(ARMA_PRINT_HDF5_ERRORS)
             {
             // Restore HDF5 error handler.
-            H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
+            arma_H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
             }
           #endif
           
@@ -2086,14 +2086,14 @@ diskio::load_hdf5_binary(Mat<eT>& x, const std::string& name, std::string& err_m
         x.set_size(dims[1], dims[0]);
         
         // Now we have to see what type is stored to figure out how to load it.
-        hid_t datatype = H5Dget_type(dataset);
+        hid_t datatype = arma_H5Dget_type(dataset);
         hid_t mat_type = hdf5_misc::get_hdf5_type<eT>();
         
         // If these are the same type, it is simple.
-        if(H5Tequal(datatype, mat_type) > 0)
+        if(arma_H5Tequal(datatype, mat_type) > 0)
           {
           // Load directly; H5S_ALL used so that we load the entire dataset.
-          hid_t read_status = H5Dread(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, void_ptr(x.memptr()));
+          hid_t read_status = arma_H5Dread(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, void_ptr(x.memptr()));
           
           if(read_status >= 0) { load_okay = true; }
           }
@@ -2106,14 +2106,14 @@ diskio::load_hdf5_binary(Mat<eT>& x, const std::string& name, std::string& err_m
           }
         
         // Now clean up.
-        H5Tclose(datatype);
-        H5Tclose(mat_type);
-        H5Sclose(filespace);
+        arma_H5Tclose(datatype);
+        arma_H5Tclose(mat_type);
+        arma_H5Sclose(filespace);
         }
       
-      H5Dclose(dataset);
+      arma_H5Dclose(dataset);
     
-      H5Fclose(fid);
+      arma_H5Fclose(fid);
 
       if(load_okay == false)
         {
@@ -2128,7 +2128,7 @@ diskio::load_hdf5_binary(Mat<eT>& x, const std::string& name, std::string& err_m
     #if !defined(ARMA_PRINT_HDF5_ERRORS)
       {
       // Restore HDF5 error handler.
-      H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
+      arma_H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
       }
     #endif
 
@@ -2159,7 +2159,7 @@ diskio::load_auto_detect(Mat<eT>& x, const std::string& name, std::string& err_m
   
   #if defined(ARMA_USE_HDF5)
     // We're currently using the C bindings for the HDF5 library, which don't support C++ streams
-    if( H5Fis_hdf5(name.c_str()) ) { return load_hdf5_binary(x, name, err_msg); }
+    if( arma_H5Fis_hdf5(name.c_str()) ) { return load_hdf5_binary(x, name, err_msg); }
   #endif
 
   std::fstream f;
@@ -2472,6 +2472,9 @@ inline
 bool
 diskio::load_coord_ascii(SpMat<eT>& x, std::istream& f, std::string& err_msg)
   {
+  arma_extra_debug_sigprint();
+  arma_ignore(err_msg);
+  
   bool load_okay = f.good();
   
   f.clear();
@@ -2694,6 +2697,9 @@ inline
 bool
 diskio::load_coord_ascii(SpMat< std::complex<T> >& x, std::istream& f, std::string& err_msg)
   {
+  arma_extra_debug_sigprint();
+  arma_ignore(err_msg);
+  
   bool load_okay = f.good();
   
   f.clear();
@@ -3035,7 +3041,31 @@ diskio::load_arma_binary(SpMat<eT>& x, std::istream& f, std::string& err_msg)
     f.read( reinterpret_cast<char*>(access::rwp(x.row_indices)), std::streamsize(x.n_nonzero*sizeof(uword))  );
     f.read( reinterpret_cast<char*>(access::rwp(x.col_ptrs)),    std::streamsize((x.n_cols+1)*sizeof(uword)) );
     
-    load_okay = f.good();
+    bool check1 = true;
+    
+    for(uword i=0; i < x.n_nonzero; ++i)
+      {
+      if(x.values[i] == eT(0))  { check1 = false; break; }
+      }
+    
+    bool check2 = true;
+    
+    for(uword i=0; i < x.n_cols; ++i)
+      {
+      if(x.col_ptrs[i+1] < x.col_ptrs[i])  { check2 = false; break; }
+      }
+    
+    const bool check3 = (x.col_ptrs[x.n_cols] == x.n_nonzero);
+    
+    if((check1 == false) || (check2 == false) || (check3 == false))
+      {
+      load_okay = false;
+      err_msg = "inconsistent data in ";
+      }
+    else
+      {
+      load_okay = f.good();
+      }
     }
   else
     {
@@ -3116,7 +3146,7 @@ diskio::save_raw_ascii(const Cube<eT>& x, std::ostream& f)
           f.width(cell_width);
           }
         
-        f << x.at(row,col,slice);
+        arma_ostream::print_elem(f, x.at(row,col,slice), false);
         }
         
       f.put('\n');
@@ -3245,7 +3275,7 @@ diskio::save_arma_ascii(const Cube<eT>& x, std::ostream& f)
           f.width(cell_width);
           }
         
-        f << x.at(row,col,slice);
+        arma_ostream::print_elem(f, x.at(row,col,slice), false);
         }
       
       f.put('\n');
@@ -3326,7 +3356,7 @@ diskio::save_hdf5_binary(const Cube<eT>& x, const std::string& final_name)
     #if !defined(ARMA_PRINT_HDF5_ERRORS)
       {
       // Disable annoying HDF5 error messages.
-      H5Eset_auto(H5E_DEFAULT, NULL, NULL);
+      arma_H5Eset_auto(H5E_DEFAULT, NULL, NULL);
       }
     #endif
 
@@ -3335,7 +3365,7 @@ diskio::save_hdf5_binary(const Cube<eT>& x, const std::string& final_name)
     const std::string tmp_name = diskio::gen_tmp_name(final_name);
 
     // Set up the file according to HDF5's preferences
-    hid_t file = H5Fcreate(tmp_name.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+    hid_t file = arma_H5Fcreate(tmp_name.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
 
     // We need to create a dataset, datatype, and dataspace
     hsize_t dims[3];
@@ -3343,7 +3373,7 @@ diskio::save_hdf5_binary(const Cube<eT>& x, const std::string& final_name)
     dims[1] = x.n_cols;
     dims[0] = x.n_slices;
 
-    hid_t dataspace = H5Screate_simple(3, dims, NULL);   // treat the cube as a 3d array dataspace
+    hid_t dataspace = arma_H5Screate_simple(3, dims, NULL);   // treat the cube as a 3d array dataspace
     hid_t datatype  = hdf5_misc::get_hdf5_type<eT>();
 
     // If this returned something invalid, well, it's time to crash.
@@ -3352,15 +3382,15 @@ diskio::save_hdf5_binary(const Cube<eT>& x, const std::string& final_name)
     // MATLAB forces the users to specify a name at save time for HDF5; Octave
     // will use the default of 'dataset' unless otherwise specified, so we will
     // use that.
-    hid_t dataset = H5Dcreate(file, "dataset", datatype, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    hid_t dataset = arma_H5Dcreate(file, "dataset", datatype, dataspace, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
 
-    herr_t status = H5Dwrite(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, x.mem);
+    herr_t status = arma_H5Dwrite(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, x.mem);
     save_okay = (status >= 0);
 
-    H5Dclose(dataset);
-    H5Tclose(datatype);
-    H5Sclose(dataspace);
-    H5Fclose(file);
+    arma_H5Dclose(dataset);
+    arma_H5Tclose(datatype);
+    arma_H5Sclose(dataspace);
+    arma_H5Fclose(file);
 
     if(save_okay == true) { save_okay = diskio::safe_rename(tmp_name, final_name); }
 
@@ -3656,16 +3686,16 @@ diskio::load_hdf5_binary(Cube<eT>& x, const std::string& name, std::string& err_
     #if !defined(ARMA_PRINT_HDF5_ERRORS)
       {
       // Save old error handler.
-      H5Eget_auto(H5E_DEFAULT, &old_func, &old_client_data);
+      arma_H5Eget_auto(H5E_DEFAULT, &old_func, &old_client_data);
 
       // Disable annoying HDF5 error messages.
-      H5Eset_auto(H5E_DEFAULT, NULL, NULL);
+      arma_H5Eset_auto(H5E_DEFAULT, NULL, NULL);
       }
     #endif
 
     bool load_okay = false;
 
-    hid_t fid = H5Fopen(name.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
+    hid_t fid = arma_H5Fopen(name.c_str(), H5F_ACC_RDONLY, H5P_DEFAULT);
 
     if(fid >= 0)
       {
@@ -3680,27 +3710,27 @@ diskio::load_hdf5_binary(Cube<eT>& x, const std::string& name, std::string& err_
 
       if(dataset >= 0)
         {
-        hid_t filespace = H5Dget_space(dataset);
+        hid_t filespace = arma_H5Dget_space(dataset);
 
         // This must be <= 3 due to our search rules.
-        const int ndims = H5Sget_simple_extent_ndims(filespace);
+        const int ndims = arma_H5Sget_simple_extent_ndims(filespace);
 
         hsize_t dims[3];
-        const herr_t query_status = H5Sget_simple_extent_dims(filespace, dims, NULL);
+        const herr_t query_status = arma_H5Sget_simple_extent_dims(filespace, dims, NULL);
 
         // arma_check(query_status < 0, "Cube::load(): cannot get size of HDF5 dataset");
         if(query_status < 0)
           {
           err_msg = "cannot get size of HDF5 dataset in ";
 
-          H5Sclose(filespace);
-          H5Dclose(dataset);
-          H5Fclose(fid);
+          arma_H5Sclose(filespace);
+          arma_H5Dclose(dataset);
+          arma_H5Fclose(fid);
 
           #if !defined(ARMA_PRINT_HDF5_ERRORS)
             {
             // Restore HDF5 error handler.
-            H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
+            arma_H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
             }
           #endif
 
@@ -3713,14 +3743,14 @@ diskio::load_hdf5_binary(Cube<eT>& x, const std::string& name, std::string& err_
         x.set_size(dims[2], dims[1], dims[0]);
 
         // Now we have to see what type is stored to figure out how to load it.
-        hid_t datatype = H5Dget_type(dataset);
+        hid_t datatype = arma_H5Dget_type(dataset);
         hid_t mat_type = hdf5_misc::get_hdf5_type<eT>();
 
         // If these are the same type, it is simple.
-        if(H5Tequal(datatype, mat_type) > 0)
+        if(arma_H5Tequal(datatype, mat_type) > 0)
           {
           // Load directly; H5S_ALL used so that we load the entire dataset.
-          hid_t read_status = H5Dread(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, void_ptr(x.memptr()));
+          hid_t read_status = arma_H5Dread(dataset, datatype, H5S_ALL, H5S_ALL, H5P_DEFAULT, void_ptr(x.memptr()));
 
           if(read_status >= 0) { load_okay = true; }
           }
@@ -3733,14 +3763,14 @@ diskio::load_hdf5_binary(Cube<eT>& x, const std::string& name, std::string& err_
           }
 
         // Now clean up.
-        H5Tclose(datatype);
-        H5Tclose(mat_type);
-        H5Sclose(filespace);
+        arma_H5Tclose(datatype);
+        arma_H5Tclose(mat_type);
+        arma_H5Sclose(filespace);
         }
 
-      H5Dclose(dataset);
+      arma_H5Dclose(dataset);
 
-      H5Fclose(fid);
+      arma_H5Fclose(fid);
 
       if(load_okay == false)
         {
@@ -3755,7 +3785,7 @@ diskio::load_hdf5_binary(Cube<eT>& x, const std::string& name, std::string& err_
     #if !defined(ARMA_PRINT_HDF5_ERRORS)
       {
       // Restore HDF5 error handler.
-      H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
+      arma_H5Eset_auto(H5E_DEFAULT, old_func, old_client_data);
       }
     #endif
 
@@ -3786,7 +3816,7 @@ diskio::load_auto_detect(Cube<eT>& x, const std::string& name, std::string& err_
   
   #if defined(ARMA_USE_HDF5)
     // We're currently using the C bindings for the HDF5 library, which don't support C++ streams
-    if( H5Fis_hdf5(name.c_str()) ) { return load_hdf5_binary(x, name, err_msg); }
+    if( arma_H5Fis_hdf5(name.c_str()) ) { return load_hdf5_binary(x, name, err_msg); }
   #endif
 
   std::fstream f;
