@@ -22,6 +22,9 @@ template<typename eT> class Cube;
 template<typename eT> class xvec_htrans;
 template<typename oT> class field;
 
+template<typename eT, bool do_conj> class xtrans_mat;
+
+
 template<typename eT> class subview;
 template<typename eT> class subview_col;
 template<typename eT> class subview_row;
@@ -66,6 +69,16 @@ class op_diagvec;
 class op_vectorise_col;
 class op_normalise_colvec;
 class op_normalise_rowvec;
+class op_clamp;
+class op_cumsum_vec;
+class op_shuffle;
+class op_sort;
+class op_find;
+class op_find_simple;
+class op_flipud;
+class op_fliplr;
+class op_real;
+class op_imag;
 
 class eop_conj;
 
